/*
 * uart1.c
 *
 * Created 14 Nov 2013
 * Author: Joel Heck
 * 
 * Adapted from code "uart1.c" created by Brian on 30 Mar 2012 for a 4760
 * project
 * 
 * Defines functions for using UART0
 */

#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include "uart_test.h"

// Structure defined in main.h for passing received data to gps.c
int_func data_receive1;
// Current index in data buffer that is being sent
static uint8_t send_pos1;
// Length of the data buffer
static uint8_t send_num1;
// Buffer for sending data
static char* send_buf1;


/*
 * Data received interrupt
 * 
 * Sends the received byte to the function receive_gps_uart() (gps.c)
 */
ISR(USART0_RX_vect)
{
	char c;
	c = UDR0;	// Store received data, UDR0 is RX/TX data buffer

	data_receive1(c);	
} 

/*
 * Data buffer empty interrupt 
 * 
 * Triggers when UDR1 is empty, meaning data has been sent. If still sending
 * data (send_num1 > send_pos1), load char from buffer into UDR1 to send it.
 * Must write new data or disable UDRIE1 to stop interrupt
 */
ISR(USART0_UDRE_vect)
{
	// Continue sending data if available
	if(send_num1 > send_pos1){
		UDR0= send_buf1[send_pos1];
		send_pos1++;
	} else {
		send_num1= 0;
		// Disable interrupt to stop sending data
		UCSR0B &= ~(1 << UDRIE0); 
	}
} 
 

/*
 * Initializes UART1 to 9600 baud, transmit and receieve, 8N1.
 * (See page 180 in datasheet for UART initialization example)
 * 
 * receive_function: pointer to function to receive data from UART1
 */
void uart1_init(int_func receive_function){

	// "gps.c" function for receiving data
	data_receive1= receive_function;
	send_pos1= 0;
	send_num1= 0;

	// Turn on the UART1 TX and RX
	UCSR0B |= (1 << RXEN0) | (1 << TXEN0);  
	// Use 8-bit character size, no parity, one stop bit (8N1)
	UCSR0C |= (1 << UCSZ10) | (1 << UCSZ11); 

	// Load upper 8-bits of the baud rate value into high byte of UBRR register
	UBRR0H = (BAUD0_PRESCALE >> 8); 
	// Load lower 8-bits of the baud rate value into low byte of UBRR register
	UBRR0L = BAUD0_PRESCALE; 

	// Enable the USART Recieve Complete interrupt (USART_RXC)
	UCSR0B |= (1 << RXCIE0); 
	//Disable sending
	UCSR0B &= ~(1 << UDRIE0); 
}

/*
 * Puts data into the send buffer and enables UART1 for sending data
 * 
 * buffer: the char array of data to send
 * num: the length of "buffer"
 *
 * return: status code (-1 = all data sent, 0 = more data to send)
 */
int8_t uart1_send_buff(char* buffer, uint8_t num){
	// Do not send if already sending data
	if(send_num1)
		return -1;
	else {
		send_num1= num;
		send_pos1= 0;
		send_buf1= buffer;
		//enable UART interrupt (Data Register Empty Interrupt will execute)
		UCSR0B |= (1 << UDRIE0);

		return 0;
	}
}

/*
 * Returns an integer depending on whether the UART1 interface is busy or not
 * 
 * return: status code (-1 = busy, 0 = not busy)
 */
int8_t uart1_busy(void){
	if(send_num1 > 0)
		return -1;
	else
		return 0;
}
